@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2005, 2007, 2009  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007, 2009, 2011, 2012  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2001  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_tasks.c,v 1.42 2009-01-22 23:47:54 tbox Exp $ */
+/* $Id$ */
 
 #include <config.h>
 
@@ -68,6 +68,7 @@ t1_callback(isc_task_t *task, isc_event_t *event) {
 
 static void
 t1_shutdown(isc_task_t *task, isc_event_t *event) {
+
 	UNUSED(task);
 
 	t_info("shutdown %s\n", (char *)event->ev_arg);
@@ -76,6 +77,7 @@ t1_shutdown(isc_task_t *task, isc_event_t *event) {
 
 static void
 my_tick(isc_task_t *task, isc_event_t *event) {
+
 	UNUSED(task);
 
 	t_info("%s\n", (char *)event->ev_arg);
@@ -400,7 +402,7 @@ t2_shutdown(isc_task_t *task, isc_event_t *event) {
 
 	isc_result_t	isc_result;
 
-	task = task; /* notused */
+	UNUSED(task);
 
 	if (event->ev_arg != NULL) {
 		isc_task_destroy((isc_task_t**) &event->ev_arg);
@@ -594,7 +596,8 @@ static	int		T3_nprobs;
 
 static void
 t3_sde1(isc_task_t *task, isc_event_t *event) {
-	task = task;
+
+	UNUSED(task);
 
 	if (T3_nevents != T3_NEVENTS) {
 		t_info("Some events were not processed\n");
@@ -612,7 +615,7 @@ t3_sde1(isc_task_t *task, isc_event_t *event) {
 static void
 t3_sde2(isc_task_t *task, isc_event_t *event) {
 
-	task = task;
+	UNUSED(task);
 
 	if (T3_nevents != T3_NEVENTS) {
 		t_info("Some events were not processed\n");
@@ -631,7 +634,7 @@ static void
 t3_event1(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T3_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -654,7 +657,7 @@ t3_event1(isc_task_t *task, isc_event_t *event) {
 
 static void
 t3_event2(isc_task_t *task, isc_event_t *event) {
-	task = task;
+	UNUSED(task);
 
 	++T3_nevents;
 	isc_event_free(&event);
@@ -733,7 +736,7 @@ t_tasks3(void) {
 	if (isc_result != ISC_R_SUCCESS) {
 		t_info("isc_task_create failed %s\n",
 		       isc_result_totext(isc_result));
-		isc_mutex_unlock(&T3_mx);
+		(void) isc_mutex_unlock(&T3_mx);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
@@ -744,6 +747,14 @@ t_tasks3(void) {
 	 */
 	event = isc_event_allocate(mctx, &senders[1], event_type, t3_event1,
 				   NULL, sizeof(*event));
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		(void) isc_mutex_unlock(&T3_mx);
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event);
 
 	/*
@@ -752,6 +763,14 @@ t_tasks3(void) {
 	for (cnt = 0; cnt < T3_NEVENTS; ++cnt) {
 		event = isc_event_allocate(mctx, &senders[1], event_type,
 					   t3_event2, NULL, sizeof(*event));
+		if (event == NULL) {
+			t_info("isc_event_allocate failed\n");
+			(void) isc_mutex_unlock(&T3_mx);
+			isc_task_destroy(&task);
+			isc_taskmgr_destroy(&tmgr);
+			isc_mem_destroy(&mctx);
+			return(T_UNRESOLVED);
+		}
 		isc_task_send(task, &event);
 	}
 
@@ -762,7 +781,7 @@ t_tasks3(void) {
 	if (isc_result != ISC_R_SUCCESS) {
 		t_info("isc_task_send failed %s\n",
 				isc_result_totext(isc_result));
-		isc_mutex_unlock(&T3_mx);
+		(void) isc_mutex_unlock(&T3_mx);
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
@@ -773,7 +792,7 @@ t_tasks3(void) {
 	if (isc_result != ISC_R_SUCCESS) {
 		t_info("isc_task_send failed %s\n",
 				isc_result_totext(isc_result));
-		isc_mutex_unlock(&T3_mx);
+		(void) isc_mutex_unlock(&T3_mx);
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
@@ -788,14 +807,14 @@ t_tasks3(void) {
 	T3_flag = 1;
 	isc_result = isc_condition_signal(&T3_cv);
 	if (isc_result != ISC_R_SUCCESS) {
-		t_info("isc_task_send failed %s\n",
+		t_info("isc_condition_signal failed %s\n",
 				isc_result_totext(isc_result));
 		++T3_nprobs;
 	}
 
 	isc_result = isc_mutex_unlock(&T3_mx);
 	if (isc_result != ISC_R_SUCCESS) {
-		t_info("isc_task_send failed %s\n",
+		t_info("isc_mutex_unlock failed %s\n",
 				isc_result_totext(isc_result));
 		++T3_nprobs;
 	}
@@ -810,11 +829,6 @@ t_tasks3(void) {
 		++T3_nfails;
 	}
 
-	if (T3_nevents != T3_nevents) {
-		t_info("T3_nevents == %d, expected 2\n", T3_nevents);
-		++T3_nfails;
-	}
-
 	result = T_UNRESOLVED;
 
 	if (T3_nfails != 0)
@@ -897,7 +911,6 @@ t_tasks4(void) {
 	T4_nfails = 0;
 	T4_flag = 0;
 
-	result = T_UNRESOLVED;
 	event_type = 4;
 
 	workers = 2;
@@ -936,7 +949,7 @@ t_tasks4(void) {
 		t_info("isc_taskmgr_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T4_mx);
-		isc_condition_destroy(&T4_cv);
+		(void) isc_condition_destroy(&T4_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -946,7 +959,7 @@ t_tasks4(void) {
 		t_info("isc_mutex_lock failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T4_mx);
-		isc_condition_destroy(&T4_cv);
+		(void) isc_condition_destroy(&T4_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
@@ -958,7 +971,7 @@ t_tasks4(void) {
 		t_info("isc_task_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T4_mx);
-		isc_condition_destroy(&T4_cv);
+		(void) isc_condition_destroy(&T4_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
@@ -969,6 +982,15 @@ t_tasks4(void) {
 	 */
 	event = isc_event_allocate(mctx, &senders[1], event_type, t4_event1,
 				   NULL, sizeof(*event));
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		DESTROYLOCK(&T4_mx);
+		isc_task_destroy(&task);
+		(void) isc_condition_destroy(&T4_cv);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event);
 
 	isc_task_shutdown(task);
@@ -1002,7 +1024,7 @@ t_tasks4(void) {
 	isc_task_detach(&task);
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
-	isc_condition_destroy(&T4_cv);
+	(void) isc_condition_destroy(&T4_cv);
 	DESTROYLOCK(&T4_mx);
 
 	result = T_UNRESOLVED;
@@ -1097,7 +1119,6 @@ t_tasks7(void) {
 	T7_sdflag = 0;
 	T7_eflag = 0;
 
-	result = T_UNRESOLVED;
 	event_type = 7;
 
 	workers = 2;
@@ -1136,7 +1157,7 @@ t_tasks7(void) {
 		t_info("isc_taskmgr_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_mem_destroy(&mctx);
 		return(T_UNRESOLVED);
 	}
@@ -1146,7 +1167,7 @@ t_tasks7(void) {
 		t_info("isc_mutex_lock failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_FAIL);
@@ -1158,7 +1179,7 @@ t_tasks7(void) {
 		t_info("isc_task_create failed %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		return(T_FAIL);
@@ -1169,7 +1190,7 @@ t_tasks7(void) {
 		t_info("isc_task_onshutdown returned %s\n",
 		       isc_result_totext(isc_result));
 		DESTROYLOCK(&T7_mx);
-		isc_condition_destroy(&T7_cv);
+		(void) isc_condition_destroy(&T7_cv);
 		isc_task_destroy(&task);
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
@@ -1178,6 +1199,15 @@ t_tasks7(void) {
 
 	event = isc_event_allocate(mctx, &senders[1], event_type, t7_event1,
 				   NULL, sizeof(*event));
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		DESTROYLOCK(&T7_mx);
+		(void) isc_condition_destroy(&T7_cv);
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event);
 
 	isc_task_shutdown(task);
@@ -1191,7 +1221,7 @@ t_tasks7(void) {
 			t_info("isc_time_nowplusinterval failed %s\n",
 			       isc_result_totext(isc_result));
 			DESTROYLOCK(&T7_mx);
-			isc_condition_destroy(&T7_cv);
+			(void) isc_condition_destroy(&T7_cv);
 			isc_task_destroy(&task);
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
@@ -1203,7 +1233,7 @@ t_tasks7(void) {
 			t_info("isc_condition_waituntil returned %s\n",
 			       isc_result_totext(isc_result));
 			DESTROYLOCK(&T7_mx);
-			isc_condition_destroy(&T7_cv);
+			(void) isc_condition_destroy(&T7_cv);
 			isc_task_destroy(&task);
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
@@ -1221,7 +1251,7 @@ t_tasks7(void) {
 	isc_task_detach(&task);
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
-	isc_condition_destroy(&T7_cv);
+	(void) isc_condition_destroy(&T7_cv);
 	DESTROYLOCK(&T7_mx);
 
 	result = T_UNRESOLVED;
@@ -1274,7 +1304,7 @@ static void
 t10_event1(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T10_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -1309,7 +1339,7 @@ t10_event2(isc_task_t *task, isc_event_t *event) {
 	int	type_match;
 	int	tag_match;
 
-	task = task;
+	UNUSED(task);
 
 	sender_match = 0;
 	type_match = 0;
@@ -1365,7 +1395,7 @@ static void
 t10_sde(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T10_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -1465,7 +1495,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		       isc_result_totext(isc_result));
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1478,7 +1508,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1491,7 +1521,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1502,6 +1532,16 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 	event = isc_event_allocate(mctx, (void *)1, (isc_eventtype_t)T_CONTROL,
 				   t10_event1, NULL, sizeof(*event));
 
+	if (event == NULL) {
+		t_info("isc_event_allocate failed\n");
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		DESTROYLOCK(&T10_mx);
+		(void) isc_condition_destroy(&T10_cv);
+		++*nprobs;
+		return;
+	}
 	isc_task_send(task, &event);
 
 	/*
@@ -1519,6 +1559,17 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 					    (isc_eventtype_t)(type + type_cnt),
 					    t10_event2, NULL, sizeof(*event));
 
+				if (eventtab[event_cnt] == NULL) {
+					t_info("isc_event_allocate failed\n");
+					isc_task_destroy(&task);
+					isc_taskmgr_destroy(&tmgr);
+					isc_mem_destroy(&mctx);
+					DESTROYLOCK(&T10_mx);
+					(void) isc_condition_destroy(&T10_cv);
+					++*nprobs;
+					return;
+				}
+
 				eventtab[event_cnt]->ev_tag =
 					(void *)((uintptr_t)tag + tag_cnt);
 
@@ -1577,7 +1628,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T10_mx);
-		isc_condition_destroy(&T10_cv);
+		(void) isc_condition_destroy(&T10_cv);
 		++*nprobs;
 		return;
 	}
@@ -1610,7 +1661,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
 			DESTROYLOCK(&T10_mx);
-			isc_condition_destroy(&T10_cv);
+			(void) isc_condition_destroy(&T10_cv);
 			++*nprobs;
 			return;
 		}
@@ -1623,7 +1674,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 			isc_taskmgr_destroy(&tmgr);
 			isc_mem_destroy(&mctx);
 			DESTROYLOCK(&T10_mx);
-			isc_condition_destroy(&T10_cv);
+			(void) isc_condition_destroy(&T10_cv);
 			++*nfails;
 			return;
 		}
@@ -1640,7 +1691,7 @@ t_taskpurge_x(int sender, int type, int tag, void *purge_sender,
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
 	DESTROYLOCK(&T10_mx);
-	isc_condition_destroy(&T10_cv);
+	(void) isc_condition_destroy(&T10_cv);
 
 	if (T_debug)
 		t_info("task processed %d events\n", T10_eventcnt);
@@ -1732,7 +1783,7 @@ static void
 t11_event1(isc_task_t *task, isc_event_t *event) {
 	isc_result_t	isc_result;
 
-	task = task;
+	UNUSED(task);
 
 	isc_result = isc_mutex_lock(&T11_mx);
 	if (isc_result != ISC_R_SUCCESS) {
@@ -1857,7 +1908,7 @@ t_tasks11(int purgable) {
 		       isc_result_totext(isc_result));
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T11_mx);
-		isc_condition_destroy(&T11_cv);
+		(void) isc_condition_destroy(&T11_cv);
 		return(T_UNRESOLVED);
 	}
 
@@ -1869,7 +1920,7 @@ t_tasks11(int purgable) {
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T11_mx);
-		isc_condition_destroy(&T11_cv);
+		(void) isc_condition_destroy(&T11_cv);
 		return(T_UNRESOLVED);
 	}
 
@@ -1881,7 +1932,7 @@ t_tasks11(int purgable) {
 		isc_taskmgr_destroy(&tmgr);
 		isc_mem_destroy(&mctx);
 		DESTROYLOCK(&T11_mx);
-		isc_condition_destroy(&T11_cv);
+		(void) isc_condition_destroy(&T11_cv);
 		return(T_UNRESOLVED);
 	}
 
@@ -1890,11 +1941,28 @@ t_tasks11(int purgable) {
 	 */
 	event1 = isc_event_allocate(mctx, (void *)1, (isc_eventtype_t)1,
 				    t11_event1, NULL, sizeof(*event1));
-
+	if (event1 == NULL) {
+		t_info("isc_event_allocate failed\n");
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		DESTROYLOCK(&T11_mx);
+		(void) isc_condition_destroy(&T11_cv);
+		return(T_UNRESOLVED);
+	}
 	isc_task_send(task, &event1);
 
 	event2 = isc_event_allocate(mctx, (void *)1, (isc_eventtype_t)1,
 				    t11_event2, NULL, sizeof(*event2));
+	if (event2 == NULL) {
+		t_info("isc_event_allocate failed\n");
+		isc_task_destroy(&task);
+		isc_taskmgr_destroy(&tmgr);
+		isc_mem_destroy(&mctx);
+		DESTROYLOCK(&T11_mx);
+		(void) isc_condition_destroy(&T11_cv);
+		return(T_UNRESOLVED);
+	}
 	event2_clone = event2;
 	if (purgable)
 		event2->ev_attributes &= ~ISC_EVENTATTR_NOPURGE;
@@ -1964,7 +2032,7 @@ t_tasks11(int purgable) {
 	isc_taskmgr_destroy(&tmgr);
 	isc_mem_destroy(&mctx);
 	DESTROYLOCK(&T11_mx);
-	isc_condition_destroy(&T11_cv);
+	(void) isc_condition_destroy(&T11_cv);
 
 	if (T11_eventcnt != (purgable ? 0 : 1)) {
 		t_info("Event was %s purged\n",
